@@ -10,11 +10,13 @@
 #' \item \code{db = "refseq"}
 #' \item \code{db = "genbank"}
 #' \item \code{db = "ensembl"}
-#' \item \code{db = "ensemblgenomes"}
 #' } 
 #' @param organism a character string specifying the scientific name of the 
 #' organism of interest, e.g. \code{organism = "Homo sapiens"}.
 #' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.
+#' @param release the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
+#' that the most recent database version is used.
+#' @param gunzip a logical value indicating whether or not files should be unzipped.
 #' @param path a character string specifying the location (a folder) in which 
 #' the corresponding annotation file shall be stored. Default is 
 #' \code{path = file.path("_ncbi_downloads","genomes")}.
@@ -48,24 +50,30 @@
 #' }
 #' 
 #' @seealso \code{\link{getProteome}}, \code{\link{getCDS}}, 
-#' \code{\link{getGenome}}, \code{\link{getRNA}}, \code{\link{meta.retrieval}}
+#' \code{\link{getGenome}}, \code{\link{getRNA}}, \code{\link{getRepeatMasker}}, 
+#' \code{\link{getAssemblyStats}}, \code{\link{meta.retrieval}}
 #' @export
 
 getGFF <-
     function(db = "refseq",
              organism,
-             reference = TRUE,
+             reference = FALSE,
+             release = NULL,
+             gunzip = FALSE,
              path = file.path("_ncbi_downloads", "annotation")) {
         
-       if (!is.element(db, c("refseq", "genbank", "ensembl", "ensemblgenomes")))
+       if (!is.element(db, c("refseq", "genbank", "ensembl")))
             stop(
                 "Please select one of the available data bases: 'refseq',
-                'genbank', 'ensembl', 'ensemblgenomes'."
+                'genbank', or 'ensembl'."
             )
         
         if (db == "ensemblgenomes") {
             organism_name <- is.genome.available(db = db, organism = organism, details = TRUE)$display_name[1]
-            message("Starting GFF retrieval of '", organism_name, "' from ", db, " ...")
+            if (!is.na(organism_name))
+                    message("Starting GFF retrieval of '", organism_name, "' from ", db, " ...")
+            if (is.na(organism_name))
+                    message("Starting GFF retrieval of '", organism, "' from ", db, " ...")
             message("\n")
         } else {
             message("Starting GFF retrieval of '", organism, "' from ", db, " ...")
@@ -211,7 +219,7 @@ getGFF <-
                                 )
                             )
                             
-                           message("GFF download is completed!")
+                            message("GFF download of ", organism, " is completed!")
                                 
                             # download md5checksum file for organism of interest
                             custom_download(
@@ -231,7 +239,7 @@ getGFF <-
                             file_name <- NULL
                             
                             md5_sum <- dplyr::filter(md5_file,
-                                            file_name == paste0("./", paste0(
+                                            file_name == paste0(" ./", paste0(
                                             basename(FoundOrganism$ftp_path),
                                                          "_genomic.gff.gz"
                                                      )))$md5
@@ -256,14 +264,16 @@ getGFF <-
                message("The md5 hash of file '", md5_file_path, "' matches!")
                             
                         }, error = function(e)
-                            stop(
-                                "The FTP site 'ftp://ftp.ncbi.nlm.nih.gov/' 
-                                cannot be reached. Are you connected to the 
-                                internet? Is the the FTP site '",
-                                download_url,
-                                "' currently available?",
+                        {
+                            warning(
+                                "The download session seems to have timed out at the FTP site '",
+                                download_url, "'. This could be due to an overload of queries to the databases.",
+                                " Please restart this function to continue the data retrieval process or wait ",
+                                "for a while before restarting this function in case your IP address was logged due to an query overload on the server side.",
                                 call. = FALSE
-                            ))
+                            )
+                            return("Not available")
+                        })
                     }
                     
                     
@@ -311,22 +321,32 @@ getGFF <-
                     
                     readr::write_tsv(doc, path = file.path(path,paste0("doc_",local.org,"_db_",db,".tsv")))
                     
-                    message(
-                        paste0(
-                            "The *.gff annotation file of '",
-                            organism,
-                            "' has been downloaded to '",
-                            path,
-                            "' and has been named '",
-                            paste0(local.org, "_genomic_", db, ".gff.gz"),
-                            "'."
-                        )
-                    )
+                    if (gunzip) {
+                            message(
+                                    paste0(
+                                            "The *.gff annnotation file of '",
+                                            organism,
+                                            "' has been downloaded to '",
+                                            path,
+                                            "' and has been named '",
+                                            paste0(local.org, "_genomic_", db, ".gff"),
+                                            "' ."
+                                    )
+                            )
+                            
+                    }
                     
-                    return(file.path(
-                        path,
-                        paste0(local.org, "_genomic_", db, ".gff.gz")
-                    ))
+                    if (gunzip) {
+                            message("Unzipping downloaded file ...")
+                            R.utils::gunzip(file.path(path,
+                                                      paste0(local.org, "_genomic_", db, ".gff.gz")), destname = file.path(path,
+                                                                                                                                    paste0(local.org, "_genomic_", db, ".gff")))
+                            return(file.path(path,
+                                             paste0(local.org, "_genomic_", db, ".gff")))
+                    } else {
+                            return(file.path(path,
+                                             paste0(local.org, "_genomic_", db, ".gff.gz")))
+                    }
                 } else {
                     stop(
                         "File: ",
@@ -345,11 +365,16 @@ getGFF <-
             }
             
             # download genome sequence from ENSEMBL
-            genome.path <-
-                getENSEMBL.Annotation(organism, type = "dna", 
-                                      id.type = "toplevel", path)
+                genome.path <-
+                        getENSEMBL.Annotation(
+                                organism,
+                                type = "dna",
+                                id.type = "toplevel",
+                                release = release,
+                                path = path
+                        )
             
-            if (is.logical(genome.path)) {
+            if (is.logical(genome.path[1])) {
                 if (!genome.path)
                     return(FALSE)
             } else {
@@ -408,7 +433,9 @@ getGFF <-
                     paste0("doc_", new.organism, "_db_", db, ".txt")
                 ))
                 
-                cat(paste0("File Name: ", genome.path))
+                cat(paste0("File Name: ", genome.path[1]))
+                cat("\n")
+                cat(paste0("Download Path: ", genome.path[2]))
                 cat("\n")
                 cat(paste0("Organism Name: ", new.organism))
                 cat("\n")
@@ -442,7 +469,8 @@ getGFF <-
                 sink()
                 
                 doc <- tibble::tibble(
-                    file_name = genome.path,
+                    file_name = genome.path[1],
+                    download_path = genome.path[2],
                     organism = new.organism,
                     database = db,
                     download_data = date(),
@@ -459,19 +487,41 @@ getGFF <-
                     paste0("doc_", new.organism, "_db_", db, ".tsv"))
                 )
                 
-                message(
-                    paste0(
-                        "The *.gff annotation file of '",
-                        organism,
-                        "' has been downloaded to '",
-                        genome.path,
-                        "' and has been named '",
-                        basename(genome.path),
-                        "'."
-                    )
-                )
+                if (!gunzip) {
+                        message(
+                                paste0(
+                                        "The GFF of '",
+                                        ensembl_summary$display_name[1],
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(genome.path[1]),
+                                        "'."
+                                )
+                        )
+                }
                 
-                return(genome.path)
+                if (gunzip) {
+                        message(
+                                paste0(
+                                        "The GFF of '",
+                                        ensembl_summary$display_name[1],
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(unlist(stringr::str_replace(genome.path[1], "[.]gz", ""))),
+                                        "'."
+                                )
+                        )
+                }
+                
+                if (gunzip) {
+                        message("Unzipping downloaded file ...")
+                        R.utils::gunzip(genome.path[1], destname = unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                        return(unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                } else {
+                        return(genome.path[1])
+                }
             }
         }
         
@@ -482,12 +532,17 @@ getGFF <-
             }
             
             # download genome sequence from ENSEMBLGENOMES
-            genome.path <-
-                getENSEMBLGENOMES.Annotation(organism, type = "dna", 
-                                             id.type = "toplevel", path)
+                genome.path <-
+                        getENSEMBLGENOMES.Annotation(
+                                organism,
+                                type = "dna",
+                                id.type = "toplevel",
+                                release = release,
+                                path = path
+                        )
             
-            if (is.logical(genome.path)) {
-                if (!genome.path)
+            if (is.logical(genome.path[1])) {
+                if (!genome.path[1])
                     return(FALSE)
             } else {
                 
@@ -545,7 +600,9 @@ getGFF <-
                     paste0("doc_", new.organism, "_db_", db, ".txt")
                 ))
                 
-                cat(paste0("File Name: ", genome.path))
+                cat(paste0("File Name: ", genome.path[1]))
+                cat("\n")
+                cat(paste0("Download Path: ", genome.path[2]))
                 cat("\n")
                 cat(paste0("Organism Name: ", new.organism))
                 cat("\n")
@@ -579,7 +636,8 @@ getGFF <-
                 sink()
                 
                 doc <- tibble::tibble(
-                    file_name = genome.path,
+                    file_name = genome.path[1],
+                    download_path = genome.path[2],
                     organism = new.organism,
                     database = db,
                     download_data = date(),
@@ -596,22 +654,44 @@ getGFF <-
                     paste0("doc_", new.organism, "_db_", db, ".tsv"))
                 )
                 
-                message(
-                    paste0(
-                        "The *.gff annotation file of '",
-                        organism,
-                        "' has been downloaded to '",
-                        genome.path,
-                        "' and has been named '",
-                        basename(genome.path),
-                        "'."
-                    )
-                )
+                if (!gunzip) {
+                        message(
+                                paste0(
+                                        "The GFF annotation file of '",
+                                        ensembl_summary$display_name,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(genome.path[1]),
+                                        "'."
+                                )
+                        )
+                }
                 
-                return(genome.path)
+                if (gunzip) {
+                        message(
+                                paste0(
+                                        "The GFF annotation file of '",
+                                        ensembl_summary$display_name,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(unlist(stringr::str_replace(genome.path[1], "[.]gz", ""))),
+                                        "'."
+                                )
+                        )
+                }
+                
+                if (gunzip) {
+                        message("Unzipping downloaded file ...")
+                        R.utils::gunzip(genome.path[1], destname = unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                        return(unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                } else {
+                        return(genome.path[1])
+                }
+            }
          }
     }
-}
 
 
 
