@@ -3,14 +3,16 @@
 #' By specifying the scientific name of an organism of interest the 
 #' corresponding fasta-file storing the genome of the organism of interest
 #' can be downloaded and stored locally. Genome files can be retrieved from 
-#' several databases.
+#' several databases. In addition, the genome summary statistics for the 
+#' retrieved species is stored locally to provide users with 
+#' insights regarding the genome assembly quality (see \code{\link{summary_genome}} for details).
+#' This is useful when comparing genomes with large difference in genome assembly qualities.
 #' @param db a character string specifying the database from which the genome 
 #' shall be retrieved:
 #' \itemize{
 #' \item \code{db = "refseq"}
 #' \item \code{db = "genbank"}
 #' \item \code{db = "ensembl"}
-#' \item \code{db = "ensemblgenomes"}
 #' }
 #' @param organism there are three options to characterize an organism: 
 #' \itemize{
@@ -18,7 +20,10 @@
 #' \item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
 #' \item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
 #' }
-#' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome. 
+#' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.
+#' @param release the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
+#' that the most recent database version is used.  
+#' @param gunzip a logical value indicating whether or not files should be unzipped.
 #' @param path a character string specifying the location (a folder) in which 
 #' the corresponding genome shall be stored. Default is 
 #' \code{path} = \code{file.path("_ncbi_downloads","genomes")}.
@@ -55,21 +60,24 @@
 #' Ath_genome <- read_genome(file_path, format = "fasta")
 #' }
 #' 
-#' @seealso \code{\link{getProteome}}, \code{\link{getCDS}}, 
-#' \code{\link{getGFF}}, \code{\link{getRNA}}, \code{\link{meta.retrieval}}, 
-#' \code{\link{read_genome}}
+#' @seealso \code{\link{getGenomeSet}}, \code{\link{getProteome}}, \code{\link{getCDS}}, 
+#' \code{\link{getGFF}}, \code{\link{getRNA}}, \code{\link{getRepeatMasker}}, 
+#' \code{\link{getAssemblyStats}}, \code{\link{summary_genome}}, 
+#' \code{\link{meta.retrieval}}, \code{\link{meta.retrieval.all}}, \code{\link{read_genome}}
 #' @export
 
 getGenome <-
     function(db = "refseq",
              organism,
-             reference = TRUE,
+             reference = FALSE,
+             release = NULL,
+             gunzip = FALSE,
              path = file.path("_ncbi_downloads", "genomes")) {
         
-       if (!is.element(db, c("refseq", "genbank", "ensembl", "ensemblgenomes")))
+       if (!is.element(db, c("refseq", "genbank", "ensembl")))
             stop(
                 "Please select one of the available data bases: 'refseq',
-                'genbank', 'ensembl', or 'ensemblgenomes'.",
+                'genbank', or 'ensembl'.",
                 call. = FALSE
             )
         
@@ -79,7 +87,12 @@ getGenome <-
                 
         if (db == "ensemblgenomes") {
             organism_name <- is.genome.available(db = db, organism = organism, details = TRUE)$display_name[1]
-            message("Starting genome retrieval of '", organism_name, "' from ", db, " ...")
+            
+            if (!is.na(organism_name))
+                    message("Starting genome retrieval of '", organism_name, "' from ", db, " ...")
+            if (is.na(organism_name))
+                    message("Starting genome retrieval of '", organism, "' from ", db, " ...")
+            
             message("\n")
         } else {
             message("Starting genome retrieval of '", organism, "' from ", db, " ...")
@@ -231,7 +244,7 @@ getGenome <-
                                 mode = "wb"
                             )
                             
-                            message("Genome download is completed!")
+                            message("Genome download of ", organism, " is completed!")
                             
                             # test check sum
                             md5_file_path <- file.path(path, 
@@ -243,13 +256,15 @@ getGenome <-
                             file_name <- NULL
                             
                             md5_sum <- dplyr::filter(md5_file,
-                                       file_name == paste0("./", paste0(
+                                       file_name == paste0(" ./", paste0(
                                            basename(FoundOrganism$ftp_path),
                                            "_genomic.fna.gz"
                                        )))$md5
                             
                             message("Checking md5 hash of file: ", 
                                     md5_file_path , " ...")
+                            
+                            
                             if (!(tools::md5sum(file.path(
                                 path,
                                 paste0(local.org, "_genomic_", db, ".fna.gz")
@@ -262,15 +277,18 @@ getGenome <-
                                         collapse = ""
                                     )
                                 )
-                            unlink(md5_file_path)
+                   unlink(md5_file_path)         
             message("The md5 hash of file '", md5_file_path, "' matches!")
-                            
-                        }, error = function(e)
-                            stop(
-                                "The FTP site 'ftp://ftp.ncbi.nlm.nih.gov/' cannot be reached. Are you connected to the internet? Is the the FTP site '",
-                                download_url, "' currently available?",
+                        }, error = function(e) {
+                            warning(
+                                "The download session seems to have timed out at the FTP site '",
+                                download_url, "'. This could be due to an overload of queries to the databases.",
+                                " Please restart this function to continue the data retrieval process or wait ",
+                                "for a while before restarting this function in case your IP address was logged due to an query overload on the server side.",
                                 call. = FALSE
-                            ))
+                            )
+                            return("Not available")
+                            })
                     }
                     
                     docFile(
@@ -315,24 +333,60 @@ getGenome <-
                             
                     )
                     
-                    readr::write_tsv(doc, path = file.path(path,paste0("doc_",local.org,"_db_",db,".tsv")))
+                    readr::write_tsv(doc, path = file.path(path, paste0("doc_",local.org,"_db_",db,".tsv")))
                     
-                    message(
-                        paste0(
-                            "The genome of '",
-                            organism,
-                            "' has been downloaded to '",
-                            path,
-                            "' and has been named '",
-                            paste0(local.org, "_genomic_", db, ".fna.gz"),
-                            "'."
-                        )
-                    )
+                    genome_summary_stats <- summary_genome(file = file.path(path,
+                                                                            paste0(local.org, "_genomic_", db, ".fna.gz")), organism = organism)
                     
-                    return(file.path(
-                        path,
-                        paste0(local.org, "_genomic_", db, ".fna.gz")
-                    ))
+                    readr::write_tsv(genome_summary_stats, path = file.path(path, paste0("doc_",local.org,"_db_",db,"_summary_statistics.tsv")))
+                    
+                    if (!gunzip) {
+                            message(
+                                    paste0(
+                                            "The genome of '",
+                                            organism,
+                                            "' has been downloaded to '",
+                                            path,
+                                            "' and has been named '",
+                                            paste0(local.org, "_genomic_", db, ".fna.gz"),
+                                            "'."
+                                    )
+                            )
+                    }
+                    
+                    if (gunzip) {
+                            message(
+                                    paste0(
+                                            "The genome of '",
+                                            organism,
+                                            "' has been downloaded to '",
+                                            path,
+                                            "' and has been named '",
+                                            paste0(local.org, "_genomic_", db, ".fna"),
+                                            "'."
+                                    )
+                            )
+                    }
+                    
+                    if (gunzip) {
+                            message("Unzipping downloaded file ...")
+                            R.utils::gunzip(file.path(path,
+                                                      paste0(
+                                                              local.org, "_genomic_", db, ".fna.gz"
+                                                      )), destname = file.path(path,
+                                                                               paste0(
+                                                                                       local.org, "_genomic_", db, ".fna"
+                                                                               )))
+                            return(file.path(path,
+                                             paste0(
+                                                     local.org, "_genomic_", db, ".fna"
+                                             )))
+                    } else {
+                            return(file.path(path,
+                                             paste0(
+                                                     local.org, "_genomic_", db, ".fna.gz"
+                                             )))
+                    }
                 } else {
                     stop(
                         "File: ",
@@ -352,12 +406,17 @@ getGenome <-
             }
             
             # download genome sequence from ENSEMBL
-            genome.path <-
-                getENSEMBL.Seq(organism, type = "dna", 
-                               id.type = "toplevel", path = path)
+                genome.path <-
+                        getENSEMBL.Seq(
+                                organism,
+                                type = "dna",
+                                id.type = "toplevel",
+                                release = release,
+                                path = path
+                        )
             
-            if (is.logical(genome.path)) {
-                if (!genome.path)
+            if (is.logical(genome.path[1])) {
+                if (!genome.path[1])
                     return(FALSE)
             } else {
                 
@@ -417,7 +476,9 @@ getGenome <-
                     paste0("doc_", new.organism, "_db_", db, ".txt")
                 ))
                 
-                cat(paste0("File Name: ", genome.path))
+                cat(paste0("File Name: ", genome.path[1]))
+                cat("\n")
+                cat(paste0("Download Path: ", genome.path[2]))
                 cat("\n")
                 cat(paste0("Organism Name: ", new.organism))
                 cat("\n")
@@ -451,7 +512,8 @@ getGenome <-
                 sink()
                 
                 doc <- tibble::tibble(
-                        file_name = genome.path,
+                        file_name = genome.path[1],
+                        download_path = genome.path[2],
                         organism = new.organism,
                         database = db,
                         download_data = date(),
@@ -468,19 +530,41 @@ getGenome <-
                         paste0("doc_", new.organism, "_db_", db, ".tsv"))
                         )
                 
-                message(
-                    paste0(
-                        "The genome of '",
-                        organism,
-                        "' has been downloaded to '",
-                        path,
-                        "' and has been named '",
-                        basename(genome.path),
-                        "'."
-                    )
-                )
+                if (!gunzip) {
+                        message(
+                                paste0(
+                                        "The genome of '",
+                                        organism,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(genome.path[1]),
+                                        "'."
+                                )
+                        )  
+                }
                 
-                return(genome.path)
+                if (gunzip) {
+                        message(
+                                paste0(
+                                        "The genome of '",
+                                        organism,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(unlist(stringr::str_replace(genome.path[1], "[.]gz", ""))),
+                                        "'."
+                                )
+                        )  
+                }
+                
+                if (gunzip) {
+                        message("Unzipping downloaded file ...")
+                        R.utils::gunzip(genome.path[1], destname = unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                        return(unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                } else {
+                        return(genome.path[1])
+                }
             }
         }
         
@@ -491,12 +575,15 @@ getGenome <-
             }
             
             # download genome sequence from ENSEMBLGENOMES
-            genome.path <-
-                getENSEMBLGENOMES.Seq(organism, type = "dna", 
-                                      id.type = "toplevel", path = path)
+                genome.path <-
+                        getENSEMBLGENOMES.Seq(organism,
+                                              release = release,
+                                              type = "dna",
+                                              id.type = "toplevel",
+                                              path = path)
             
-            if (is.logical(genome.path)) {
-                if (!genome.path)
+            if (is.logical(genome.path[1])) {
+                if (!genome.path[1])
                     return(FALSE)
             } else {
                 
@@ -556,7 +643,9 @@ getGenome <-
                     paste0("doc_", new.organism, "_db_", db, ".txt")
                 ))
                 
-                cat(paste0("File Name: ", genome.path))
+                cat(paste0("File Name: ", genome.path[1]))
+                cat("\n")
+                cat(paste0("Download Path: ", genome.path[2]))
                 cat("\n")
                 cat(paste0("Organism Name: ", new.organism))
                 cat("\n")
@@ -590,7 +679,8 @@ getGenome <-
                 sink()
                 
                 doc <- tibble::tibble(
-                        file_name = genome.path,
+                        file_name = genome.path[1],
+                        download_path = genome.path[2],
                         organism = new.organism,
                         database = db,
                         download_data = date(),
@@ -607,19 +697,41 @@ getGenome <-
                         paste0("doc_", new.organism, "_db_", db, ".tsv"))
                 )
                 
-                message(
-                    paste0(
-                        "The genome of '",
-                        organism,
-                        "' has been downloaded to '",
-                        path,
-                        "' and has been named '",
-                        basename(genome.path),
-                        "'."
-                    )
-                )
+                if (!gunzip) {
+                        message(
+                                paste0(
+                                        "The genome of '",
+                                        organism,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(genome.path[1]),
+                                        "'."
+                                )
+                        )  
+                }
                 
-                return(genome.path)
+                if (gunzip) {
+                        message(
+                                paste0(
+                                        "The genome of '",
+                                        organism,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(unlist(stringr::str_replace(genome.path[1], "[.]gz", ""))),
+                                        "'."
+                                )
+                        )  
+                }
+                
+                if (gunzip) {
+                        message("Unzipping downloaded file ...")
+                        R.utils::gunzip(genome.path[1], destname = unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                        return(unlist(stringr::str_replace(genome.path[1], "[.]gz", "")))
+                } else {
+                        return(genome.path[1])
+                }
         }
     }
 }
