@@ -10,7 +10,6 @@
 #' \item \code{db = "refseq"}
 #' \item \code{db = "genbank"}
 #' \item \code{db = "ensembl"}
-#' \item \code{db = "ensemblgenomes"}
 #' \item \code{db = "uniprot"}
 #' }
 #' @param organism there are three options to characterize an organism: 
@@ -20,6 +19,9 @@
 #' \item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
 #' }
 #' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.
+#' @param release the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
+#' that the most recent database version is used.
+#' @param gunzip a logical value indicating whether or not files should be unzipped.
 #' @param path a character string specifying the location (a folder) in which 
 #' the corresponding proteome shall be stored. Default is 
 #' \code{path} = \code{file.path("_ncbi_downloads","proteomes")}.
@@ -52,7 +54,8 @@
 #' Ath_proteome <- read_proteome(file_path, format = "fasta")
 #' }
 #' @seealso \code{\link{getGenome}}, \code{\link{getCDS}}, \code{\link{getGFF}},
-#' \code{\link{getRNA}}, \code{\link{meta.retrieval}}, 
+#' \code{\link{getRNA}}, \code{\link{getRepeatMasker}}, 
+#' \code{\link{getAssemblyStats}}, \code{\link{meta.retrieval}}, 
 #' \code{\link{read_proteome}}
 #' @export
 
@@ -60,19 +63,26 @@ getProteome <-
     function(db = "refseq",
              organism,
              reference = TRUE,
+             release = NULL,
+             gunzip = FALSE,
              path = file.path("_ncbi_downloads", "proteomes")) {
         if (!is.element(db, c("refseq", "genbank", 
-                              "ensembl", "ensemblgenomes", "uniprot")))
+                              "ensembl", "uniprot")))
             stop(
                 "Please select one of the available data bases: 
-                'refseq', 'genbank', 'ensembl',  'ensemblgenomes', 'uniprot'.",
+                'refseq', 'genbank', 'ensembl', 'uniprot'.",
                 call. = FALSE
             )
         
         if (db == "ensemblgenomes") {
-            organism_name <- is.genome.available(db = db, organism = organism, details = TRUE)$display_name[1]
-            message("Starting proteome retrieval of '", organism_name, "' from ", db, " ...")
-            message("\n")
+                organism_name <- is.genome.available(db = db, organism = organism, details = TRUE)$display_name[1]
+                
+                if (!is.na(organism_name))
+                        message("Starting proteome retrieval of '", organism_name, "' from ", db, " ...")
+                if (is.na(organism_name))
+                        message("Starting proteome retrieval of '", organism, "' from ", db, " ...")
+                
+                message("\n")
         } else {
             message("Starting proteome retrieval of '", organism, "' from ", db, " ...")
             message("\n")
@@ -220,7 +230,7 @@ getProteome <-
                                 )
                             )
                             
-                          message("Proteome download is completed!")
+                            message("Proteome download of ", organism, " is completed!")
                                 
                             # download md5checksum file for organism of interest
                             custom_download(
@@ -240,7 +250,7 @@ getProteome <-
                             file_name <- NULL
                             
                             md5_sum <- dplyr::filter(md5_file,
-                                            file_name == paste0("./", paste0(
+                                            file_name == paste0(" ./", paste0(
                                             basename(FoundOrganism$ftp_path),
                                                 "_protein.faa.gz"
                                             )))$md5
@@ -264,15 +274,16 @@ getProteome <-
                             unlink(md5_file_path)
                 message("The md5 hash of file '", md5_file_path, "' matches!")
                             
-                        }, error = function(e)
-                            stop(
-                                "The FTP site 'ftp://ftp.ncbi.nlm.nih.gov/' 
-                                cannot be reached. Are you connected to the 
-                                internet? Is the the FTP site '",
-                                download_url,
-                                "' currently available?",
+                        }, error = function(e){
+                            warning(
+                                "The download session seems to have timed out at the FTP site '",
+                                download_url, "'. This could be due to an overload of queries to the databases.",
+                                " Please restart this function to continue the data retrieval process or wait ",
+                                "for a while before restarting this function in case your IP address was logged due to an query overload on the server side.",
                                 call. = FALSE
-                            ))
+                            )
+                            return("Not available")
+                        })
                     }
                     
                     docFile(
@@ -317,22 +328,48 @@ getProteome <-
                     
                     readr::write_tsv(doc, path = file.path(path,paste0("doc_",local.org,"_db_",db,".tsv")))
                     
-                    message(
-                        paste0(
-                            "The proteome of '",
-                            organism,
-                            "' has been downloaded to '",
-                            path,
-                            "' and has been named '",
-                            paste0(local.org, "_protein_", db, ".faa.gz"),
-                            "' ."
-                        )
-                    )
+                    if (!gunzip) {
+                            message(
+                                    paste0(
+                                            "The proteome of '",
+                                            organism,
+                                            "' has been downloaded to '",
+                                            path,
+                                            "' and has been named '",
+                                            paste0(local.org, "_protein_", db, ".faa.gz"),
+                                            "' ."
+                                    )
+                            )
+                            
+                    }
+                    
+                    if (gunzip) {
+                            message(
+                                    paste0(
+                                            "The proteome of '",
+                                            organism,
+                                            "' has been downloaded to '",
+                                            path,
+                                            "' and has been named '",
+                                            paste0(local.org, "_protein_", db, ".faa"),
+                                            "' ."
+                                    )
+                            )
+                            
+                    }
+                    
+                    if (gunzip) {
+                            message("Unzipping downloaded file ...")
+                            R.utils::gunzip(file.path(path,
+                                                      paste0(local.org, "_protein_", db, ".faa.gz")), destname = file.path(path,
+                                                                                                                          paste0(local.org, "_protein_", db, ".faa")))
+                            return(file.path(path,
+                                             paste0(local.org, "_protein_", db, ".faa")))
+                    } else {
+                            return(file.path(path,
+                                             paste0(local.org, "_protein_", db, ".faa.gz")))
+                    }
                     
-                    return(file.path(
-                        path,
-                        paste0(local.org, "_protein_", db, ".faa.gz")
-                    ))
                 } else {
                     stop(
                         "File: ",
@@ -352,11 +389,17 @@ getProteome <-
             }
             
             # download proteome sequence from ENSEMBL
-            proteome.path <-
-                getENSEMBL.Seq(organism, type = "pep", id.type = "all", path)
+                proteome.path <-
+                        getENSEMBL.Seq(
+                                organism,
+                                type = "pep",
+                                id.type = "all",
+                                release = release,
+                                path = path
+                        )
             
-            if (is.logical(proteome.path)) {
-                if (!proteome.path)
+            if (is.logical(proteome.path[1])) {
+                if (!proteome.path[1])
                     return(FALSE)
             } else {
                 
@@ -417,7 +460,9 @@ getProteome <-
                     paste0("doc_", new.organism, "_db_", db, ".txt")
                 ))
                 
-                cat(paste0("File Name: ", proteome.path))
+                cat(paste0("File Name: ", proteome.path[1]))
+                cat("\n")
+                cat(paste0("Download Path: ", proteome.path[2]))
                 cat("\n")
                 cat(paste0("Organism Name: ", new.organism))
                 cat("\n")
@@ -451,7 +496,8 @@ getProteome <-
                 sink()
                 
                 doc <- tibble::tibble(
-                        file_name = proteome.path,
+                        file_name = proteome.path[1],
+                        download_path = proteome.path[2],
                         organism = new.organism,
                         database = db,
                         download_data = date(),
@@ -468,19 +514,41 @@ getProteome <-
                         paste0("doc_", new.organism, "_db_", db, ".tsv"))
                 )
                 
-                message(
-                    paste0(
-                        "The proteome of '",
-                        ensembl_summary$display_name[1],
-                        "' has been downloaded to '",
-                        path,
-                        "' and has been named '",
-                        basename(proteome.path),
-                        "'."
-                    )
-                )
+                if (!gunzip) {
+                        message(
+                                paste0(
+                                        "The proteome of '",
+                                        ensembl_summary$display_name[1],
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(proteome.path[1]),
+                                        "'."
+                                )
+                        )
+                }
                 
-                return(proteome.path)
+                if (gunzip) {
+                        message(
+                                paste0(
+                                        "The proteome of '",
+                                        ensembl_summary$display_name[1],
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(unlist(stringr::str_replace(proteome.path[1], "[.]gz", ""))),
+                                        "'."
+                                )
+                        )
+                }
+                
+                if (gunzip) {
+                        message("Unzipping downloaded file ...")
+                        R.utils::gunzip(proteome.path[1], destname = unlist(stringr::str_replace(proteome.path[1], "[.]gz", "")))
+                        return(unlist(stringr::str_replace(proteome.path[1], "[.]gz", "")))
+                } else {
+                        return(proteome.path[1])
+                }
             }
         }
         
@@ -491,11 +559,17 @@ getProteome <-
             }
             
             # download proteome sequence from ENSEMBLGENOMES
-            proteome.path <-
-                getENSEMBLGENOMES.Seq(organism, type = "pep", id.type = "all", path)
+                proteome.path <-
+                        getENSEMBLGENOMES.Seq(
+                                organism,
+                                type = "pep",
+                                id.type = "all",
+                                release = release,
+                                path = path
+                        )
             
-            if (is.logical(proteome.path)) {
-                if (!proteome.path)
+            if (is.logical(proteome.path[1])) {
+                if (!proteome.path[1])
                     return(FALSE)
             } else {
                 
@@ -554,7 +628,9 @@ getProteome <-
                         paste0("doc_", new.organism, "_db_", db, ".txt")
                 ))
                 
-                cat(paste0("File Name: ", proteome.path))
+                cat(paste0("File Name: ", proteome.path[1]))
+                cat("\n")
+                cat(paste0("Download Path: ", proteome.path[2]))
                 cat("\n")
                 cat(paste0("Organism Name: ", new.organism))
                 cat("\n")
@@ -588,7 +664,8 @@ getProteome <-
                 sink()
                 
                 doc <- tibble::tibble(
-                        file_name = proteome.path,
+                        file_name = proteome.path[1],
+                        download_path = proteome.path[2],
                         organism = new.organism,
                         database = db,
                         download_data = date(),
@@ -605,26 +682,50 @@ getProteome <-
                         paste0("doc_", new.organism, "_db_", db, ".tsv"))
                 )
                 
-                message(
-                    paste0(
-                        "The proteome of '",
-                        ensembl_summary$display_name,
-                        "' has been downloaded to '",
-                        path,
-                        "' and has been named '",
-                        basename(proteome.path),
-                        "'."
-                    )
-                )
+                if (!gunzip) {
+                        message(
+                                paste0(
+                                        "The proteome of '",
+                                        ensembl_summary$display_name,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(proteome.path[1]),
+                                        "'."
+                                )
+                        )
+                }
                 
-                return(proteome.path)
+                if (gunzip) {
+                        message(
+                                paste0(
+                                        "The proteome of '",
+                                        ensembl_summary$display_name,
+                                        "' has been downloaded to '",
+                                        path,
+                                        "' and has been named '",
+                                        basename(unlist(stringr::str_replace(proteome.path[1], "[.]gz", ""))),
+                                        "'."
+                                )
+                        )
+                }
+                
+                if (gunzip) {
+                        message("Unzipping downloaded file ...")
+                        R.utils::gunzip(proteome.path[1], destname = unlist(stringr::str_replace(proteome.path[1], "[.]gz", "")))
+                        return(unlist(stringr::str_replace(proteome.path[1], "[.]gz", "")))
+                } else {
+                        return(proteome.path[1])
+                }
             }
         }
         
         if (db == "uniprot") {
+                
                 if (!file.exists(path)) {
                         dir.create(path, recursive = TRUE)
                 }
+                
                 getUniProtSeq(organism = organism, path = path, update = TRUE)
         }
 
