@@ -58,7 +58,7 @@
 getRepeatMasker <-
     function(db = "refseq",
              organism,
-             reference = TRUE,
+             reference = FALSE,
              path = file.path("_ncbi_downloads", "repeatmasker")) {
         
     if (!is.element(db, c("refseq", "genbank")))
@@ -212,7 +212,7 @@ getRepeatMasker <-
                                 )
                             )
                             
-                            message("RepeatMasker download is completed!")
+                            message("RepeatMasker download of ", organism, " is completed!")
                                 
                             # download md5checksum file for organism of interest
                             custom_download(
@@ -232,7 +232,7 @@ getRepeatMasker <-
                             file_name <- NULL
                             
                             md5_sum <- dplyr::filter(md5_file,
-                                                     file_name == paste0("./", paste0(
+                                                     file_name == paste0(" ./", paste0(
                                                          basename(FoundOrganism$ftp_path),
                                                          "_rm.out.gz"
                                                      )))$md5
@@ -256,14 +256,16 @@ getRepeatMasker <-
                             message("The md5 hash of file '", md5_file_path, "' matches!")
                             
                         }, error = function(e)
-                            stop(
-                                "The FTP site 'ftp://ftp.ncbi.nlm.nih.gov/' 
-                                cannot be reached. Are you connected to the 
-                                internet? Is the the FTP site '",
-                                download_url,
-                                "' currently available?",
+                        {
+                            warning(
+                                "The download session seems to have timed out at the FTP site '",
+                                download_url, "'. This could be due to an overload of queries to the databases.",
+                                " Please restart this function to continue the data retrieval process or wait ",
+                                "for a while before restarting this function in case your IP address was logged due to an query overload on the server side.",
                                 call. = FALSE
-                            ))
+                            )
+                            return("Not available")
+                        })
                     }
                     
                     docFile(
