@@ -46,8 +46,7 @@ getSummaryFile <- function(db,kingdom){
             ))
         }, error = function(e)
             stop(
-                "The FTP site 'ftp://ftp.ncbi.nlm.nih.gov/' cannot be reached. 
-                Are you connected to the internet? Is the the FTP site '",
+                "The FTP site 'ftp://ftp.ncbi.nlm.nih.gov/' cannot be reached. Are you connected to the internet? Is the the FTP site '",
                 paste0(
                     "ftp://ftp.ncbi.nlm.nih.gov/genomes/",
                     db,
@@ -56,9 +55,9 @@ getSummaryFile <- function(db,kingdom){
                     "/assembly_summary.txt"
                 ),
                 "' currently available?",
+                " It could also be that you have a firewall issue or you previously sent many requests to NCBI and now your IP address has been blocked for the next hours.",
                 call. = FALSE
             ))
-        Sys.sleep(0.4)
     }
     
     suppressWarnings(summary.file <-
