@@ -10,10 +10,9 @@ read_md5file <- function(md5path) {
         readr::read_delim(
             md5path,
             delim = "  ",
-            col_names = c("md5", "na", "file_name"),
+            col_names = c("md5", "file_name"),
             col_types = readr::cols(
                 md5 = readr::col_character(),
-                na = readr::col_character(),
                 file_name = readr::col_character()
             )
         )
